#include "exporter.h"
#include "dialog.h"

#ifndef C4D_R96
//typedef Polygon CPolygon;
#endif



Bool Ogre3D_Exporter::TriangulateObject(BaseDocument* doc,BaseObject* object)
{
    ModelingCommandData cd;
    cd.doc=doc;
    cd.op=object;
    cd.mode=MODIFY_ALL;
    return(SendModelingCommand(MCOMMAND_TRIANGULATE,cd));
}



void Ogre3D_Exporter::CalculateNormals(PolygonObject* object)
{
    INT point,poly;
    CPolygon* polygon_array=object->GetPolygonW();
    Vector* point_array=object->GetPointW();
    normals_array=bNew Vector[3*object->GetPolygonCount()];

    // Iteration through all polygons for each point.
    // If the same vertex is in many faces, each face normal vector is added...
    for(point=0;point<object->GetPointCount();point++)
    {
        for(poly=0;poly<object->GetPolygonCount();poly++)
        {
            if(polygon_array[poly].a==point)
                normals_array[polygon_array[poly].a]+=(point_array[polygon_array[poly].b]-point_array[polygon_array[poly].a])%(point_array[polygon_array[poly].c]-point_array[polygon_array[poly].a]);

            if(polygon_array[poly].b==point)
                normals_array[polygon_array[poly].b]+=(point_array[polygon_array[poly].c]-point_array[polygon_array[poly].b])%(point_array[polygon_array[poly].a]-point_array[polygon_array[poly].b]);

            if(polygon_array[poly].c==point)
                normals_array[polygon_array[poly].c]+=(point_array[polygon_array[poly].a]-point_array[polygon_array[poly].c])%(point_array[polygon_array[poly].b]-point_array[polygon_array[poly].c]);
        }
    }

    // ... then the resulting normal is normalized
    for(point=0;point<object->GetPointCount();point++)
        normals_array[point]/=Len(normals_array[point]);
}



void Ogre3D_Exporter::PrepareObjectForUVExport(PolygonObject* object)
{
// This functions aims to avoid a point being shared by many polygons.
// This type of points are searched and duplicated in all the polygons sharing it.
// This is needed for a complete support of UV mapping.
    INT t;
    LONG point_count;
    CPolygon* polygon_array=object->GetPolygonW();
    Vector* point_array=object->GetPointW();
    BaseTag* tag;
    Real* vertex_map_data;
    Bool* point_already_met=bNew INT[object->GetPointCount()];

    for(t=0;t<object->GetPointCount();t++)
        point_already_met[t]=FALSE;

    // Iteration through all polygons of the object
    for(t=0;t<object->GetPolygonCount();t++)
    {
        // Point a
        if(!point_already_met[polygon_array[t].a])
            point_already_met[polygon_array[t].a]=TRUE;
        else
        {
            // Creation of a new point
            VariableChanged vc;
            point_count=object->GetPointCount();
            vc.old_cnt=point_count;
            vc.new_cnt=point_count+1;
            object->Message(MSG_POINTS_CHANGED,&vc);
            point_count=object->GetPointCount();
            point_array=object->GetPointW();
            point_array[point_count-1]=point_array[polygon_array[t].a];

            // Set the normal for the new point
            if(parameters.exportNormals)
                normals_array[point_count-1]=normals_array[polygon_array[t].a];

            // Set bone weighting for the new point
            tag=object->GetFirstTag();
            while(tag)
            {
                if(tag->GetType()==Tvertexmap)
                {
                        vertex_map_data=(Real*)((VariableTag*)tag)->GetDataAddressW();
                        vertex_map_data[point_count-1]=vertex_map_data[polygon_array[t].a];
                }
                tag=tag->GetNext();
            }

            polygon_array[t].a=point_count-1;
            polygon_array[t].d=polygon_array[t].c;
        }

        // Point b
        if(!point_already_met[polygon_array[t].b])
            point_already_met[polygon_array[t].b]=TRUE;
        else
        {
            // Creation of a new point
            VariableChanged vc;
            point_count=object->GetPointCount();
            vc.old_cnt=point_count;
            vc.new_cnt=point_count+1;
            object->Message(MSG_POINTS_CHANGED,&vc);
            point_count=object->GetPointCount();
            point_array=object->GetPointW();
            point_array[point_count-1]=point_array[polygon_array[t].b];

            // Set the normal for the new point
            if(parameters.exportNormals)
                normals_array[point_count-1]=normals_array[polygon_array[t].b];

            // Set bone weighting for the new point
            tag=object->GetFirstTag();
            while(tag)
            {
                if(tag->GetType()==Tvertexmap)
                {
                        vertex_map_data=(Real*)((VariableTag*)tag)->GetDataAddressW();
                        vertex_map_data[point_count-1]=vertex_map_data[polygon_array[t].b];
                }
                tag=tag->GetNext();
            }

            polygon_array[t].b=point_count-1;
            polygon_array[t].d=polygon_array[t].c;
        }

        // Point c
        if(!point_already_met[polygon_array[t].c])
            point_already_met[polygon_array[t].c]=TRUE;
        else
        {
            // Creation of a new point
            VariableChanged vc;
            point_count=object->GetPointCount();
            vc.old_cnt=point_count;
            vc.new_cnt=point_count+1;
            object->Message(MSG_POINTS_CHANGED,&vc);
            point_count=object->GetPointCount();
            point_array=object->GetPointW();
            point_array[point_count-1]=point_array[polygon_array[t].c];

            // Set the normal for the new point
            if(parameters.exportNormals)
                normals_array[point_count-1]=normals_array[polygon_array[t].c];

            // Set bone weighting for the new point
            tag=object->GetFirstTag();
            while(tag)
            {
                if(tag->GetType()==Tvertexmap)
                {
                        vertex_map_data=(Real*)((VariableTag*)tag)->GetDataAddressW();
                        vertex_map_data[point_count-1]=vertex_map_data[polygon_array[t].c];
                }
                tag=tag->GetNext();
            }

            polygon_array[t].c=point_count-1;
            polygon_array[t].d=polygon_array[t].c;
        }
    }

    // Updates the object
    object->Message(MSG_UPDATE);
    bDelete(point_already_met);
}



Vector* Ogre3D_Exporter::GetUVData(PolygonObject* object)
{
    // Get first UV tag (if at least one)
    UVWTag* uvw_tag=(UVWTag*)object->GetTag(Tuvw,0);
    if(!uvw_tag)
    {
        MessageDialog("Object \""+object->GetName()+"\" has no UVW tag.\nUV coordinates can't be exported.");
        return NULL;
    }

    // Transforming C4D's per-polygon UV array to a per-vertex UV array
    Vector* per_vertex_uv_coords=bNew Vector[object->GetPointCount()];
    CPolygon* polygon_array=object->GetPolygonW();
    INT t;
    for(t=0;t<=object->GetPolygonCount()-1;t++)
    {
        per_vertex_uv_coords[polygon_array[t].a]=uvw_tag->Get(t).a;
        per_vertex_uv_coords[polygon_array[t].b]=uvw_tag->Get(t).b;
        per_vertex_uv_coords[polygon_array[t].c]=uvw_tag->Get(t).c;
    }
    return per_vertex_uv_coords;
}



String* Ogre3D_Exporter::GetVertexMaps(SkeletonElement* skeleton_element)
{
    BaseTag* restriction_tag=skeleton_element->GetBone()->GetTag(Trestriction,0);
    if(restriction_tag)
    {
        String* vertex_maps=bNew String[6];
        BaseContainer* tag_container=restriction_tag->GetDataInstance();
        vertex_maps[0]=tag_container->GetString(RESTRICTIONTAG_NAME_01);
        vertex_maps[1]=tag_container->GetString(RESTRICTIONTAG_NAME_02);
        vertex_maps[2]=tag_container->GetString(RESTRICTIONTAG_NAME_03);
        vertex_maps[3]=tag_container->GetString(RESTRICTIONTAG_NAME_04);
        vertex_maps[4]=tag_container->GetString(RESTRICTIONTAG_NAME_05);
        vertex_maps[5]=tag_container->GetString(RESTRICTIONTAG_NAME_06);

        return vertex_maps;
    }
    else
        return NULL;
}



String Ogre3D_Exporter::GetMaterialName(BaseObject* object)
{
    // Get the first texture tag (if at least one)
    TextureTag* texture_tag=(TextureTag*)object->GetTag(Ttexture,0);

    if(!texture_tag)
    {
        MessageDialog("Object \""+object->GetName()+"\" has no texture tag.\nTexture name can't be exported.");
        return NULL;
    }
    #ifndef C4D_R9
    BaseMaterial* material=texture_tag->GetMaterial();
    #else
    BaseMaterial* material=texture_tag->GetMaterial(TRUE);
    #endif
    return material->GetName();
}



Filename* Ogre3D_Exporter::SelectFilename()
{
    Filename* fname=gNew Filename;
    String title="Save as...";
    if(fname->FileSelect(FSTYPE_ANYTHING,GE_SAVE,&title))
        return fname;
    else
    {
        gDelete(fname);
        return NULL;
    }
}



Filename* Ogre3D_Exporter::SelectPath()
{
    Filename* fname=gNew Filename;
    String title="Select a directory...";
    if(fname->FileSelect(FSTYPE_ANYTHING,GE_DIRECTORY,&title))
        return fname;
    else
    {
        gDelete(fname);
        return NULL;
    }
}



void Ogre3D_Exporter::WriteString(String string,BaseFile* file)
{
    LONG length=string.GetCStringLen(St8bit)+1;
    CHAR* to_write=(CHAR*)GeAlloc(length);
    string.GetCString(to_write,length,St8bit);
    INT t;
    for(t=0;t<length-1;t++)
        file->WriteChar(to_write[t]);
    GeFree(to_write);
}



void Ogre3D_Exporter::WriteMeshXMLFile(BaseObject* object,BaseDocument* doc,Filename* fname)
{
    Bool multiple_submeshes=FALSE;  // TRUE if object to object to export is a null object containing objects
    PolygonObject* clone_object=NULL;
    Skeleton* skeleton=NULL;

    // Get a clone object
    if(object->GetType()==Onull)
    {
        object=object->GetDown();
        if(object)
            clone_object=(PolygonObject*)object->GetClone(0,NULL);
        multiple_submeshes=TRUE;
    }
    else
        clone_object=(PolygonObject*)object->GetClone(0,NULL);

    // Open file to write
    BaseFile* file=BaseFile::Alloc();
    file->Open(*fname,GE_WRITE,FILE_DIALOG,GeGetByteOrder());

    // Begin writing
    WriteString("<mesh>\n",file);
    WriteString("\t<submeshes>\n",file);
    do
    {
        if(clone_object->GetType()==Opolygon)
        {
            if(parameters.exportSkeleton)
                skeleton=gNew Skeleton(clone_object);

            TriangulateObject(doc,clone_object);
            if(parameters.exportNormals)
                CalculateNormals(clone_object);
            if(parameters.exportUV)
                PrepareObjectForUVExport(clone_object);
            Vector* UVData=GetUVData(clone_object);

            // Write material data
            if(parameters.includeMaterialName)
            {
                String material_name=GetMaterialName(clone_object);
                if(material_name!="")
                    WriteString("\t\t<submesh material=\""+material_name+"\" usesharedvertices=\"false\" use32bitindexes=\"false\" operationtype=\"triangle_list\">\n",file);
                else
                    WriteString("\t\t<submesh usesharedvertices=\"false\" use32bitindexes=\"false\" operationtype=\"triangle_list\">\n",file);
            }
            else
                WriteString("\t\t<submesh usesharedvertices=\"false\" use32bitindexes=\"false\" operationtype=\"triangle_list\">\n",file);

            // Write polygon data
            WriteString("\t\t\t<faces count=\""+LongToString(clone_object->GetPolygonCount())+"\">\n",file);
            INT t;
            CPolygon* polygon_array=clone_object->GetPolygonW();
            for(t=0;t<clone_object->GetPolygonCount();t++)
            {
                WriteString("\t\t\t\t<face v1=\""+LongToString(polygon_array[t].a)+"\" v2=\""+LongToString(polygon_array[t].b)+"\" v3=\""+LongToString(polygon_array[t].c)+"\" />\n",file);
            }
            WriteString("\t\t\t</faces>\n",file);

            // Write vertex data
            WriteString("\t\t\t<geometry vertexcount=\""+LongToString(clone_object->GetPointCount())+"\">\n",file);
            String normals_string;
            if(parameters.exportNormals)
                normals_string="\"true\"";
            else
                normals_string="\"false\"";
            if(parameters.exportUV && UVData)
                WriteString("\t\t\t\t<vertexbuffer positions=\"true\" normals="+normals_string+" texture_coords=\"1\" texture_coords_dimensions_0=\"2\">\n",file);
            else
                WriteString("\t\t\t\t<vertexbuffer positions=\"true\" normals="+normals_string+">\n",file);

            Vector* point_array=clone_object->GetPointW();
            Vector point;
            for(t=0;t<clone_object->GetPointCount();t++)
            {
                point=point_array[t];
                WriteString("\t\t\t\t\t<vertex>\n",file);
                // Position
                WriteString("\t\t\t\t\t\t<position x=\""+RealToString(point.x)+"\" y=\""+RealToString(point.y)+"\" z=\""+RealToString(point.z)+"\" />\n",file);
                // Normal
                if(parameters.exportNormals)
                    WriteString("\t\t\t\t\t\t<normal x=\""+RealToString(normals_array[t].x)+"\" y=\""+RealToString(normals_array[t].y)+"\" z=\""+RealToString(normals_array[t].z)+"\" />\n",file);
                // UV coordinates
                if(parameters.exportUV && UVData)
                    WriteString("\t\t\t\t\t\t<texcoord u=\""+RealToString(UVData[t].x)+"\" v=\""+RealToString(UVData[t].y)+"\" />\n",file);
                WriteString("\t\t\t\t\t</vertex>\n",file);
            }
            bDelete(normals_array);
            bDelete(UVData);
            WriteString("\t\t\t\t</vertexbuffer>\n",file);
            WriteString("\t\t\t</geometry>\n",file);

            // Write bone assignments
            if(parameters.exportSkeleton)
            {
                if(skeleton->GetFirstSkeletonElement())
                {
                    WriteString("\t\t\t<boneassignments>\n",file);
                    String* vertex_maps;
                    INT point;
                    INT bone_index;
                    Real* vertex_map_data;

                    for(bone_index=0;bone_index<skeleton->GetSkeletonElementNumber();bone_index++)
                    {
                        skeleton->ToIndex(bone_index);
                        if(vertex_maps=GetVertexMaps(skeleton->GetCurrentSkeletonElement()))
                        {
                            for(t=0;t<=5;t++)
                            {
                                if(vertex_maps[t]!="")
                                {
                                    BaseTag* tag=(VariableTag*)clone_object->GetFirstTag();
                                    while(tag!=NULL)
                                    {
                                        if(tag->GetName()==vertex_maps[t] && tag->GetType()==Tvertexmap)
                                        {
                                            vertex_map_data=(Real*)((VariableTag*)tag)->GetDataAddressW();
                                            for(point=0;point<clone_object->GetPointCount();point++)
                                                WriteString("\t\t\t\t<vertexboneassignment vertexindex=\""+LongToString(point)+"\" boneindex=\""+LongToString(bone_index)+"\" weight=\""+RealToString(vertex_map_data[point])+"\" />\n",file);
                                        }
                                        tag=tag->GetNext();
                                    }
                                }
                            }
                            bDelete(vertex_maps);
                        }
                        else
                        {
                            MessageDialog("There is no restriction tag assigned to \""+skeleton->GetCurrentSkeletonElement()->GetBone()->GetName()+"\". Skeleton won't be exported.");
                            parameters.exportSkeleton=FALSE;
                        }
                    }
                    WriteString("\t\t\t</boneassignments>\n",file);
                }
            }

            WriteString("\t\t</submesh>\n",file);
            // End submesh writing

            gDelete(skeleton);
        }
        object=object->GetNext();
        if(object)
            clone_object=(PolygonObject*)object->GetClone(0,NULL); // Get a clone of the next submesh if one
    }
    while(object && multiple_submeshes);

    WriteString("\t</submeshes>\n",file);

    // Write skeleton link
    if(parameters.exportSkeleton)
    {
        Filename skeleton_name=fname->GetFile();
        skeleton_name.ClearSuffix();
        skeleton_name.ClearSuffix();
        WriteString("\t<skeletonlink name=\""+skeleton_name.GetString()+".skeleton\" />\n",file);
    }

    WriteString("</mesh>\n",file);
    // End mesh writing

    file->Close();
    BaseFile::Free(file);
    PolygonObject::Free(clone_object);
}



void Ogre3D_Exporter::WriteSkeletonXMLFile(BaseObject* object,BaseDocument* doc,Filename* fname)
{
    Skeleton* skeleton=gNew Skeleton(object);

    if(skeleton->GetFirstSkeletonElement())
    {
        BaseFile* file=BaseFile::Alloc();
        file->Open(*fname,GE_WRITE,FILE_DIALOG,GeGetByteOrder());
        WriteString("<skeleton>\n",file);
        WriteString("\t<bones>\n",file);
        INT t,f;
        BaseTime time,time_start,time_end;
        Matrix matrix,matrix0,old_matrix;
        Vector axis,pos;
        Real angle;

        // Write initial pose for each bone
        time.SetNominator(parameters.firstFrame);
        doc->AnimateObject(object,time,ANIMATE_NO_PARTICLES);

        for(t=0;t<skeleton->GetSkeletonElementNumber();t++)
        {
            WriteString("\t\t<bone id=\""+LongToString(t)+"\" name=\"Bone"+LongToString(t)+"\">\n",file);
            skeleton->ToIndex(t);

            // Get a relative-to-parent matrix for the current bone
            matrix0=skeleton->GetCurrentSkeletonElement()->GetBone()->GetMl();

            // Position
            WriteString("\t\t\t<position x=\""+RealToString(matrix0.off.x)+"\" y=\""+RealToString(matrix0.off.y)+"\" z=\""+RealToString(matrix0.off.z)+"\" />\n",file);

            // Convert Euler matrix to angle/axis
            MatrixToRotAxis(matrix0,&axis,&angle);

            // Rotation
            WriteString("\t\t\t<rotation angle=\""+RealToString(angle)+"\">\n",file);
            WriteString("\t\t\t\t<axis x=\""+RealToString(axis.x)+"\" y=\""+RealToString(axis.y)+"\" z=\""+RealToString(axis.z)+"\" />\n",file);
            WriteString("\t\t\t</rotation>\n",file);
            WriteString("\t\t</bone>\n",file);
        }
        WriteString("\t</bones>\n",file);

        // Write bone hierarchy
        WriteString("\t<bonehierarchy>\n",file);
        for(t=0;t<skeleton->GetSkeletonElementNumber();t++)
        {
            skeleton->ToIndex(t);
            if(skeleton->GetCurrentSkeletonElement()->GetParentSkeletonElement())
                WriteString("\t\t<boneparent bone=\"Bone"+LongToString(t)+"\" parent=\"Bone"+LongToString(skeleton->GetCurrentSkeletonElement()->GetParentSkeletonElement()->GetIndex())+"\" />\n",file);
        }
        WriteString("\t</bonehierarchy>\n",file);


        // Write animation data
        time_start.SetNominator(parameters.firstFrame);
        time_start.SetDenominator(doc->GetFps());
        time_end.SetNominator(parameters.lastFrame);
        time_end.SetDenominator(doc->GetFps());
        time.SetDenominator(doc->GetFps());
        WriteString("\t<animations>\n",file);
        WriteString("\t\t<animation name=\""+parameters.animationName+"\" length=\""+RealToString(time_end.Get()-time_start.Get())+"\">\n",file);
        WriteString("\t\t\t<tracks>\n",file);

        for(t=0;t<skeleton->GetSkeletonElementNumber();t++)
        {
            skeleton->ToIndex(t);
            time.SetNominator(parameters.firstFrame);
            doc->AnimateObject(object,time,ANIMATE_NO_PARTICLES);
            matrix0=skeleton->GetCurrentSkeletonElement()->GetBone()->GetMl();

            WriteString("\t\t\t\t<track bone=\"Bone"+LongToString(t)+"\">\n",file);
            WriteString("\t\t\t\t\t<keyframes>\n",file);

            // Iteration through all frames
            for(f=parameters.firstFrame;f<=parameters.lastFrame;f++)
            {
                // Set current time
                time.SetNominator(f);
                doc->AnimateObject(object,time,ANIMATE_NO_PARTICLES);

                // Get a relative-to-parent matrix for the current bone
                matrix=skeleton->GetCurrentSkeletonElement()->GetBone()->GetMl();

                // Write keyframe
                if(old_matrix!=matrix || f==parameters.firstFrame)
                {
                    WriteString("\t\t\t\t\t\t<keyframe time=\""+RealToString(time.Get()-time_start.Get())+"\">\n",file);
                    // Position
                    WriteString("\t\t\t\t\t\t\t<translate x=\""+RealToString(matrix.off.x-matrix0.off.x)+"\" y=\""+RealToString(matrix.off.y-matrix0.off.y)+"\" z=\""+RealToString(matrix.off.z-matrix0.off.z)+"\" />\n",file);

                    // Convert Euler matrix to angle/axis
                    MatrixToRotAxis(matrix,&axis,&angle);

                    // Rotation
                    WriteString("\t\t\t\t\t\t\t<rotate angle=\""+RealToString(angle)+"\">\n",file);
                    WriteString("\t\t\t\t\t\t\t\t<axis x=\""+RealToString(axis.x)+"\" y=\""+RealToString(axis.y)+"\" z=\""+RealToString(axis.z)+"\" />\n",file);
                    WriteString("\t\t\t\t\t\t\t</rotate>\n",file);

                    WriteString("\t\t\t\t\t\t</keyframe>\n",file);

                    old_matrix=matrix;
                }
            }
            WriteString("\t\t\t\t\t</keyframes>\n",file);
            WriteString("\t\t\t\t</track>\n",file);
        }
        WriteString("\t\t\t</tracks>\n",file);
        WriteString("\t\t</animation>\n",file);
        WriteString("\t</animations>\n",file);
        WriteString("</skeleton>\n",file);
        file->Close();
        BaseFile::Free(file);
    }
    gDelete(skeleton);
}



LONG Ogre3D_Exporter::GetState(BaseDocument* doc)
{
    return CMD_ENABLED;
}

Bool Ogre3D_Exporter::Execute(BaseDocument* doc)
{
    ExporterDialog* d=gNew ExporterDialog(doc);
    parameters=d->parameters;

    // If "Cancel" pressed
    if(parameters.cancel)
        return FALSE;

    // If scene is empty
    if(parameters.noObjects)
    {
        MessageDialog("There is no object to export.");
        return FALSE;
    }

    // Export of the whole scene
    if(parameters.exportAllObjects)
    {
        Filename* path=SelectPath();
        Filename* fname=gNew Filename;

        if(path)
        {
            BaseObject* object=doc->GetFirstObject();
            INT n_objects_exported=0;
            do
            {
                // Determine file name
                if(object->GetType()!=Opolygon && object->GetType()!=Onull)
                {
                    object=object->GetNext();
                    continue;
                }
                Filename fname2(object->GetName()+".mesh.xml");
                Filename fname3(path->GetString());
                fname->SetFile(fname2);
                fname->SetDirectory(fname3);

                // Write and convert XML files
                WriteMeshXMLFile(object,doc,fname);
                if(parameters.convertXML && parameters.converterFname.GetString()!="")
                {
                    if(!GeExecuteProgram(parameters.converterFname,*fname))
                        MessageDialog("An error occured while converting \""+fname->GetString()+"\" to binary mesh.");
                }

                if(parameters.exportSkeleton)
                {
                    Filename fname4(object->GetName()+".skeleton.xml");
                    fname->SetFile(fname4);
                    fname->SetDirectory(fname3);
                    WriteSkeletonXMLFile(object,doc,fname);
                    if(parameters.convertXML && parameters.converterFname.GetString()!="")
                    {
                        if(!GeExecuteProgram(parameters.converterFname,*fname))
                            MessageDialog("An error occured while converting \""+fname->GetString()+"\" to binary skeleton.");
                    }
                }
                object=object->GetNext();
                n_objects_exported++;
            }
            while(object);

            if(n_objects_exported>1)
                MessageDialog(LongToString(n_objects_exported)+" objects successfully exported.");
            else
                MessageDialog(LongToString(n_objects_exported)+" object successfully exported.");

            gDelete(fname);
        }
    }
    else
        // Single object export mode
    {
        Filename* fname=SelectFilename();

        if(fname)
        {
            fname->ClearSuffix();
            fname->ClearSuffix();
            fname->SetString(fname->GetString()+".mesh.xml");
            if(parameters.object->GetType()!=Opolygon && parameters.object->GetType()!=Onull)
            {
                MessageDialog("Convert \""+parameters.object->GetName()+"\" to object before exporting it.");
                return FALSE;
            }
            WriteMeshXMLFile(parameters.object,doc,fname);
            if(parameters.convertXML)
            {
                if(parameters.converterFname.GetString()!="")
                {
                    if(!GeExecuteProgram(parameters.converterFname,*fname))
                        MessageDialog("An error occured while converting \""+fname->GetString()+"\" to binary mesh.");
                    else
                        MessageDialog("Object exported successfully.");
                }
                else
                    MessageDialog("You must select the location of your converter (OgreXMLConverter.exe).");
            }
            else
                MessageDialog("Object exported successfully.");
            if(parameters.exportSkeleton)
            {
                fname->ClearSuffix();
                fname->ClearSuffix();
                fname->SetString(fname->GetString()+".skeleton.xml");
                WriteSkeletonXMLFile(parameters.object,doc,fname);
                if(parameters.convertXML)
                {
                    if(parameters.converterFname.GetString()!="")
                    {
                        if(!GeExecuteProgram(parameters.converterFname,*fname))
                            MessageDialog("An error occured while converting \""+fname->GetString()+"\" to binary skeleton.");
                        else
                            MessageDialog("Skeleton exported successfully.");
                    }
                }
                else
                    MessageDialog("Skeleton exported successfully.");
            }
            gDelete(fname);
        }
    }
    gDelete(d);
    return TRUE;
}
