#include "skeleton.h"

SkeletonElement::SkeletonElement(BaseObject* bone)
{
    _bone=bone;
    _next_skeleton_element=NULL;
    _child_skeleton_element=NULL;
    _parent_skeleton_element=NULL;
    _index=0;
}

BaseObject* SkeletonElement::GetBone()
{
    return _bone;
}

SkeletonElement* SkeletonElement::GetNextSkeletonElement()
{
    return _next_skeleton_element;
}

SkeletonElement* SkeletonElement::GetChildSkeletonElement()
{
    return _child_skeleton_element;
}

SkeletonElement* SkeletonElement::GetParentSkeletonElement()
{
    return _parent_skeleton_element;
}

INT SkeletonElement::GetIndex()
{
    return _index;
}

void SkeletonElement::SetNextSkeletonElement(SkeletonElement* skeleton_element)
{
    _next_skeleton_element=skeleton_element;
}

void SkeletonElement::SetChildSkeletonElement(SkeletonElement* skeleton_element)
{
   _child_skeleton_element=skeleton_element;
}

void SkeletonElement::SetParentSkeletonElement(SkeletonElement* skeleton_element)
{
    _parent_skeleton_element=skeleton_element;
}

void SkeletonElement::SetIndex(INT index)
{
    _index=index;
}




Skeleton::Skeleton(BaseObject* object)
{
    _first_skeleton_element=NULL;
    _current_skeleton_element=NULL;
    _n_bones=0;
    GetObjectSkeleton(object);
}

Skeleton::~Skeleton()
{
    // Delete all skeleton elements searching them by index (should be optimized)
    do
    {
        ToIndex(_n_bones-1);
        gDelete(_current_skeleton_element);
        _n_bones--;
    }
    while(_n_bones!=0);
}


SkeletonElement* Skeleton::GetFirstSkeletonElement()
{
    return _first_skeleton_element;
}

SkeletonElement* Skeleton::GetCurrentSkeletonElement()
{
    return _current_skeleton_element;
}

INT Skeleton::GetSkeletonElementNumber()
{
    return _n_bones;
}

void Skeleton::ToFirstSkeletonElement()
{
	_current_skeleton_element=_first_skeleton_element;
}

void Skeleton::ToNextSkeletonElement()
{
    _current_skeleton_element=_current_skeleton_element->GetNextSkeletonElement();
}

void Skeleton::ToChildSkeletonElement()
{
    _current_skeleton_element=_current_skeleton_element->GetChildSkeletonElement();
}

void Skeleton::ToParentSkeletonElement()
{
    _current_skeleton_element=_current_skeleton_element->GetParentSkeletonElement();
}



void Skeleton::InsertSkeletonElement(BaseObject* bone)
{
    SkeletonElement* skeleton_element=gNew SkeletonElement(bone);

    if(!_first_skeleton_element)
    {
        _first_skeleton_element=skeleton_element;
	  _current_skeleton_element=skeleton_element;
    }
    else
    {
	  SkeletonElement* t=_first_skeleton_element;
        while(t->GetNextSkeletonElement())
            t->GetNextSkeletonElement();
        t->SetNextSkeletonElement(skeleton_element);
    }
    skeleton_element->SetIndex(_n_bones);
    _n_bones++;
}




void Skeleton::InsertChildSkeletonElement(BaseObject* child_bone,SkeletonElement* parent_skeleton_element)
{
    SkeletonElement* skeleton_element=gNew SkeletonElement(child_bone);
    SkeletonElement* last_child_skeleton_element=parent_skeleton_element->GetChildSkeletonElement();

    if(last_child_skeleton_element)
    {
        while(last_child_skeleton_element->GetNextSkeletonElement())
            last_child_skeleton_element=last_child_skeleton_element->GetNextSkeletonElement();
        last_child_skeleton_element->SetNextSkeletonElement(skeleton_element);
    }

    skeleton_element->SetParentSkeletonElement(parent_skeleton_element);
    parent_skeleton_element->SetChildSkeletonElement(skeleton_element);
    skeleton_element->SetIndex(_n_bones);
    _n_bones++;
}



void Skeleton::GetObjectSkeleton(BaseObject* object)
{
    Bool go_down,go_up,go_next,finished;
    go_down=TRUE;
    go_next=FALSE;
    go_up=FALSE;
    finished=FALSE;
    BaseObject* parent_object=object;
    while(!finished)
    {
        if(go_down)
        {
            if(object->GetDown())
                object=object->GetDown();
            else
            {
                go_down=FALSE;
                go_up=FALSE;
                go_next=TRUE;
                continue;
            }

            if(object->GetType()==Obone)
            {
                if(object->GetUp()==parent_object)
                {
                    InsertSkeletonElement(object);
                    ToNextSkeletonElement();
                    if(!_current_skeleton_element)
                        ToFirstSkeletonElement();
                }
                else
                {
                    InsertChildSkeletonElement(object,_current_skeleton_element);
                    ToChildSkeletonElement();
                }
                go_down=TRUE;
                go_up=FALSE;
                go_next=FALSE;
            }
            else
            {
                go_down=FALSE;
                go_up=FALSE;
                go_next=TRUE;
            }
        }

        if(go_next)
        {
            if(object->GetNext())
                object=object->GetNext();
            else
            {
                go_down=FALSE;
                go_up=TRUE;
                go_next=FALSE;
                continue;
            }

            if(object->GetType()==Obone)
            {
                if(object->GetUp()==parent_object)
                {
                    InsertSkeletonElement(object);
                    ToNextSkeletonElement();
                    if(!_current_skeleton_element)
                        ToFirstSkeletonElement();
                }
                else
                {
                    InsertChildSkeletonElement(object,_current_skeleton_element->GetParentSkeletonElement());
                    ToNextSkeletonElement();
                }
                go_down=TRUE;
                go_up=FALSE;
                go_next=FALSE;
            }
            else
            {
                go_down=FALSE;
                go_up=FALSE;
                go_next=TRUE;
            }
        }

        if(go_up)
        {
            if(object->GetUp()==parent_object || object==parent_object)
                finished=TRUE;
            else
            {
                object=object->GetUp();
                if(_current_skeleton_element->GetParentSkeletonElement())
                    ToParentSkeletonElement();
                go_down=FALSE;
                go_up=FALSE;
                go_next=TRUE;
            }
        }
    }
}



void Skeleton::ToIndex(INT index)
{
    Bool go_down,go_up,go_next,found;
    go_down=TRUE;
    go_next=FALSE;
    go_up=FALSE;
    found=FALSE;

    if(index>_n_bones-1 || !_first_skeleton_element)
        return;
    if(_first_skeleton_element->GetIndex()==index)
    {
        _current_skeleton_element=_first_skeleton_element;
        return;
    }

    _current_skeleton_element=_first_skeleton_element;

    while(!found)
    {
        if(go_down)
        {
            if(_current_skeleton_element->GetChildSkeletonElement())
            {
                ToChildSkeletonElement();
                if(_current_skeleton_element->GetIndex()==index)
                {
                    found=TRUE;
                    continue;
                }
                go_down=TRUE;
                go_up=FALSE;
                go_next=FALSE;
            }
            else
            {
                go_down=FALSE;
                go_up=FALSE;
                go_next=TRUE;
            }
        }

        if(go_next)
        {
            if(_current_skeleton_element->GetNextSkeletonElement())
            {
                ToNextSkeletonElement();
                if(_current_skeleton_element->GetIndex()==index)
                {
                    found=TRUE;
                    continue;
                }
                go_down=TRUE;
                go_up=FALSE;
                go_next=FALSE;
            }
            else
            {
                go_down=FALSE;
                go_up=TRUE;
                go_next=FALSE;
            }
        }

        if(go_up)
        {
            ToParentSkeletonElement();
            go_down=FALSE;
            go_up=FALSE;
            go_next=TRUE;
        }
    }
}
